cmd_commands=[
"arp",
"assoc",
"attrib",
"auditpol",
"bcdboot",
"bcdedit",
"bitsadmin",
"BREAK",
"cacls",
"call",
"cd",
"certreq",
"certutil",
"change",
"changepk",
"chcp",
"chdir",
"chkdsk",
"chkntfs",
"choice",
"cipher",
"cleanmgr",
"clip",
"cls",
"cmd"
"cmdkey",
"color",
"comp",
"compact",
"convert",
"copy",
"curl",
"date",
"defrag",
"DEL",
"dir",
"diskpart",
"dism",
"displayswitch",
"doskey",
"driverquery",
"dsregcmd",
"echo",
"endlocal",
"erase",
"esentutl",
"eventcreate",
"exit",
"expand",
"explorer",
"fc",
"find",
"findstr",
"fltmc",
"FOR",
"forfiles",
"format",
"fsutil",
"ftp",
"ftype",
"getmac",
"goto",
"gpresult",
"gpupdate",
"help",
"hostname",
"icacls",
"iexpress",
"IF",
"ipconfig",
"klist",
"label",
"lodctr",
"logman",
"logoff",
"makecab",
"manage_bde",
"mbr2gpt",
"md",
"mkdir",
"mklink",
"mode",
"more",
"mountvol",
"move",
"moveuser",
"msg",
"msiexec",
"msinfo32",
"mstsc",
"nbtstat",
"net",
"netsh",
"netstat",
"nltest",
"nslookup",
"openfiles",
"path",
"pathping",
"pause",
"ping",
"pktmon",
"pnputil",
"popd",
"powercfg",
"PRINT",
"prompt",
"pushd",
"qappsrv",
"qprocess",
"quser",
"qwinsta",
"rasdial",
"rasphone",
"rd",
"reagentc",
"recover",
"reg",
"regedit",
"regini",
"regsvr32",
"ren",
"replace",
"reset",
"rmdir",
"rmtshare",
"robocopy",
"route",
"runas",
"rundll32",
"sc",
"schtasks",
"scriptrunner",
"set",
"setlocal",
"setspn",
"setx",
"sfc",
"shift",
"shutdown",
"sleep",
"slmgr",
"sort",
"ssh",
"start",
"subst",
"sxstrace",
"systeminfo",
"takeown",
"tar",
"taskkill",
"tasklist",
"time",
"timeout",
"title",
"tracert",
"tree",
"tsdiscon",
"tskill",
"type",
"typeperf",
"tzutil",
"ver",
"verify",
"vol",
"vssadmin",
"w32tm",
"waitfor",
"wbadmin",
"wecutil",
"wevtutil",
"where",
"whoami",
"windiff",
"winget",
"winmgmt",
"winrm",
"winrs",
"wmic",
"wpr",
"wt",
"wuauclt",
"wusa",
"xcopy"
]


from ..bin.cmd import cmd

def ping(arguments: str|None = None):
    """
    Envía un ping a un host y retorna el resultado.

    Args:
        arguments (str|None): Dirección IP o nombre del host al que se le envía el ping.

    Returns:
        str: Respuesta del ping (ej. "¡Pong!" o un mensaje de error).

    Raises:
        ValueError: Si el host está vacío o el timeout es negativo.
    """
    return cmd(f"ping {arguments}")


from subprocess import run
from .shell import shell_open,no_shell,no_shell_open,shell_run


def powershell(command):
        
        return shell(["powershell", "-Command", full_command],arguments)
        


from .shell import shell_open,no_shell,no_shell_open,shell_run

def cmd(command,mode="shell_open"):
    if mode == "shell_open":
        return shell_open(command)
    if mode == "no_shell":
        return no_shell(command)
    if mode == "no_shell_open":
        return no_shell_open(command)
    if mode == "shell_run":
        return shell_run(command)


archivos = [
"arp.py",
"assoc.py",
"attrib.py",
"auditpol.py",
"bcdboot.py",
"bcdedit.py",
"bitsadmin.py",
"BREAK.py",
"cacls.py",
"call.py",
"cd.py",
"certreq.py",
"certutil.py",
"change.py",
"changepk.py",
"chcp.py",
"chdir.py",
"chkdsk.py",
"chkntfs.py",
"choice.py",
"cipher.py",
"cleanmgr.py",
"clip.py",
"cls.py",
"cmdkey.py",
"color.py",
"comp.py",
"compact.py",
"convert.py",
"copy.py",
"curl.py",
"date.py",
"defrag.py",
"DEL.py",
"dir.py",
"diskpart.py",
"dism.py",
"displayswitch.py",
"doskey.py",
"driverquery.py",
"dsregcmd.py",
"echo.py",
"endlocal.py",
"erase.py",
"esentutl.py",
"eventcreate.py",
"exit.py",
"expand.py",
"explorer.py",
"fc.py",
"find.py",
"findstr.py",
"fltmc.py",
"FOR.py",
"forfiles.py",
"format.py",
"fsutil.py",
"ftp.py",
"ftype.py",
"getmac.py",
"goto.py",
"gpresult.py",
"gpupdate.py",
"help.py",
"hostname.py",
"icacls.py",
"iexpress.py",
"IF.py",
"ipconfig.py",
"klist.py",
"label.py",
"lodctr.py",
"logman.py",
"logoff.py",
"makecab.py",
"manage_bde.py",
"mbr2gpt.py",
"md.py",
"mkdir.py",
"mklink.py",
"mode.py",
"more.py",
"mountvol.py",
"move.py",
"moveuser.py",
"msg.py",
"msiexec.py",
"msinfo32.py",
"mstsc.py",
"nbtstat.py",
"net.py",
"netsh.py",
"netstat.py",
"nltest.py",
"nslookup.py",
"openfiles.py",
"path.py",
"pathping.py",
"pause.py",
"ping.py",
"pktmon.py",
"pnputil.py",
"popd.py",
"powercfg.py",
"PRINT.py",
"prompt.py",
"pushd.py",
"qappsrv.py",
"qprocess.py",
"quser.py",
"qwinsta.py",
"rasdial.py",
"rasphone.py",
"rd.py",
"reagentc.py",
"recover.py",
"reg.py",
"regedit.py",
"regini.py",
"regsvr32.py",
"ren.py",
"replace.py",
"reset.py",
"rmdir.py",
"rmtshare.py",
"robocopy.py",
"route.py",
"runas.py",
"rundll32.py",
"sc.py",
"schtasks.py",
"scriptrunner.py",
"set.py",
"setlocal.py",
"setspn.py",
"setx.py",
"sfc.py",
"shift.py",
"shutdown.py",
"sleep.py",
"slmgr.py",
"sort.py",
"ssh.py",
"start.py",
"subst.py",
"sxstrace.py",
"systeminfo.py",
"takeown.py",
"tar.py",
"taskkill.py",
"tasklist.py",
"time.py",
"timeout.py",
"title.py",
"tracert.py",
"tree.py",
"tsdiscon.py",
"tskill.py",
"type.py",
"typeperf.py",
"tzutil.py",
"ver.py",
"verify.py",
"vol.py",
"vssadmin.py",
"w32tm.py",
"waitfor.py",
"wbadmin.py",
"wecutil.py",
"wevtutil.py",
"where.py",
"whoami.py",
"windiff.py",
"winget.py",
"winmgmt.py",
"winrm.py",
"winrs.py",
"wmic.py",
"wpr.py",
"wt.py",
"wuauclt.py",
"wusa.py",
"xcopy.py"
]


# Crear un nuevo archivo llamado "mi_archivo.txt" y escribir en él

for archivo in archivos:
    archivo_sin_extencion = archivo.split(".",1)
    content = f"""
from ..bin.cmd import cmd

def {archivo_sin_extencion}(arguments=None):
    command = [
        "{archivo_sin_extencion}",
        arguments
    ]
    return cmd(command)
    """
    with open(archivo, "w") as file:
        file.write(content)


from .command_list_cmd import cmd_commands

def _validateArguments(arguments):
    if arguments == "None":
        return False
    if arguments == None:
        return False
    return True

def _validateCommand(command):
    for _command in cmd_commands:
        if command == _command:
            return True
        return False
    



from ..bin.cmd import cmd

def copy(source = "",destination = "",sourceArguments = "",destinationArguments = ""):
    arguments = f"{source} {sourceArguments} {destination} {destinationArguments}"
    return cmd("copy",arguments)


from subprocess import run,Popen,PIPE
from ..models.Command import Command
from sys import stdin,stderr
from threading import Thread

#TODO: crear un validador para command, 
#   1 verifica el tipo, puede ser string y lista,
#   2 verifica que no este vacio|None tanto la string como la lista 

def shell_run(command):
    process = run(command,
                    capture_output=True,
                    text=True,
                    shell=True
                    )
    result = Command(
        output=process.stdout,
        error=process.stderr,
        command=process.args,
        returncode=process.returncode,
        process_identifier=0
    )
    return result

def shell_open(command):
    # Inicia el proceso hijo
    process = Popen(
        command,
        stdin=stdin,          # Permite interacción desde el teclado
        stdout=PIPE,   # Captura stdout
        stderr=PIPE,   # Captura stderr
        text=True,                # Trabaja con texto (no bytes)
        bufsize=1,                # Buffering línea por línea
        shell=True
    )
    
    # Variables para almacenar output
    stdout_lines = []
    stderr_lines = []

    # ---- Función anidada 1: Para imprimir stdout en vivo ----
    def print_stdout():
        for line in process.stdout:   # type: ignore # Lee línea por línea del pipe
            print(line, end="")    # Muestra en tiempo real
            stdout_lines.append(line)

    # ---- Función anidada 2: Para imprimir stderr en vivo ----
    def print_stderr():
        for line in process.stderr:   # Lee línea por línea del pipe #type: ignore
            print(line, end="", file=stderr)  # Muestra errores en stderr
            stderr_lines.append(line)

    # ---- Inicia hilos para leer stdout y stderr sin bloquear ----
    stdout_thread = Thread(target=print_stdout)
    stderr_thread = Thread(target=print_stderr)
    
    stdout_thread.start()
    stderr_thread.start()

    # Espera a que el proceso termine
    process.wait()
    
    # muestra lo que quedo de la salida y de los errores.
    stdout_thread.join()
    stderr_thread.join()
    
    result = Command(
        output="".join(stdout_lines),
        error="".join(stderr_lines),
        command=command if isinstance(command, str) else " ".join(command),
        returncode=process.returncode,
        process_identifier=process.pid
        )
    
    return result

def no_shell_open(command: str|list):
    # Inicia el proceso hijo
    process = Popen(
        command,
        stdin=stdin,          # Permite interacción desde el teclado
        stdout=PIPE,   # Captura stdout
        stderr=PIPE,   # Captura stderr
        text=True,                # Trabaja con texto (no bytes)
        bufsize=1                # Buffering línea por línea
    )
    
    # Variables para almacenar output
    stdout_lines = []
    stderr_lines = []

    # ---- Función anidada 1: Para imprimir stdout en vivo ----
    def print_stdout():
        for line in process.stdout:   # type: ignore # Lee línea por línea del pipe
            print(line, end="")    # Muestra en tiempo real
            stdout_lines.append(line)

    # ---- Función anidada 2: Para imprimir stderr en vivo ----
    def print_stderr():
        for line in process.stderr:   # Lee línea por línea del pipe #type: ignore
            print(line, end="", file=stderr)  # Muestra errores en stderr
            stderr_lines.append(line)

    # ---- Inicia hilos para leer stdout y stderr sin bloquear ----
    stdout_thread = Thread(target=print_stdout)
    stderr_thread = Thread(target=print_stderr)
    
    stdout_thread.start()
    stderr_thread.start()

    # Espera a que el proceso termine
    process.wait()
    
    # muestra lo que quedo de la salida y de los errores.
    stdout_thread.join()
    stderr_thread.join()
    
    result = Command(
        output="".join(stdout_lines),
        error="".join(stderr_lines),
        command=command if isinstance(command, str) else " ".join(command),
        returncode=process.returncode,
        process_identifier=process.pid
        )
    
    return result
    
def no_shell(command: str|list):
    process = run(command,
                    capture_output=True,
                    text=True
                    )
    
    result = Command(
        output=process.stdout,
        error=process.stderr,
        command=process.args,
        returncode=process.returncode,
        process_identifier=0
    )
    
    return result